@@ -32,7 +32,6 @@
 #include <libbonobo.h>
 #include <bonobo/bonobo-property-bag-client.h>
 #include <glade/glade-init.h>
-#include <monkey-media.h>
 #include <stdlib.h>
 #include <unistd.h>
 #include <time.h>
@@ -40,6 +39,10 @@
 #ifdef HAVE_GSTREAMER
 #include <gst/gst.h>
 #include <gst/gconf/gconf.h>
+#include <gst/control/control.h>
+#endif
+#ifdef WITH_MONKEYMEDIA
+#include "monkey-media.h"
 #endif
 
 #include "rb-shell.h"
@@ -52,7 +55,6 @@
 #include "eel-gconf-extensions.h"
 
 static gboolean rb_init (RBShell *shell);
-static gboolean sound_error_dialog (gpointer unused);
 static void rb_handle_cmdline (char **argv, int argc,
 			       gboolean already_running);
 
@@ -61,13 +63,16 @@ static CORBA_Environment ev;
 static gboolean debug           = FALSE;
 static gboolean quit            = FALSE;
 static gboolean no_registration = FALSE;
+static gboolean no_update	= FALSE;
 static gboolean dry_run		= FALSE;
+static char *rhythmdb_file = NULL;
 static gboolean print_playing = FALSE;
 static gboolean print_playing_path = FALSE;
 static gboolean playpause       = FALSE;
 static gboolean focus           = FALSE;
 static gboolean previous        = FALSE;
 static gboolean next            = FALSE;
+static gboolean shuffle         = FALSE;
 static gboolean print_play_time = FALSE;
 static gboolean print_song_length = FALSE;
 static long seek_time           = 0;
@@ -78,7 +83,7 @@ main (int argc, char **argv)
 	GnomeProgram *program;
 	CORBA_Object object;
 	RBShell *rb_shell;
-	gboolean sound_events_borked = FALSE;
+	char **new_argv;
 
 	const struct poptOption popt_options[] =
 	{
@@ -94,17 +99,27 @@ main (int argc, char **argv)
 		{ "previous",		    0,  POPT_ARG_NONE,			&previous,	        	0, N_("Jump to previous song"),     NULL },
 		{ "next",		        0,  POPT_ARG_NONE,			&next,		        	0, N_("Jump to next song"),     NULL },
 		
+		{ "shuffle",		        0,  POPT_ARG_NONE,			&shuffle,		        	0, N_("Toggle shuffling"),     NULL },
+
 		{ "debug",           'd',  POPT_ARG_NONE,          &debug,                                        0, N_("Enable debugging code"),     NULL },
+		{ "no-update", 0,  POPT_ARG_NONE,          &no_update,                              0, N_("Do not update the library"), NULL },
 		{ "no-registration", 'n',  POPT_ARG_NONE,          &no_registration,                              0, N_("Do not register the shell"), NULL },
 		{ "dry-run", 0,  POPT_ARG_NONE,          &dry_run,                             0, N_("Don't save any data permanently (implies --no-registration)"), NULL },
+		{ "rhythmdb-file", 0,  POPT_ARG_STRING,          &rhythmdb_file,                             0, N_("Path for database file to use"), NULL },
 		{ "quit",            'q',  POPT_ARG_NONE,          &quit,                                         0, N_("Quit Rhythmbox"),            NULL },
-		{ NULL,              '\0', POPT_ARG_INCLUDE_TABLE, (poptOption *) monkey_media_get_popt_table (), 0, N_("MonkeyMedia options:"),      NULL },
 		POPT_TABLEEND
 	};
 
+	/* Disable event sounds for now by passing "--disable-sound" to libgnomeui.
+	 * See: http://bugzilla.gnome.org/show_bug.cgi?id=119222 */
+	new_argv = g_strdupv (argv);
+	new_argv = g_realloc (new_argv, (argc+2)*sizeof(char*));
+	new_argv[argc] = g_strdup ("--disable-sound");
+	new_argv[argc+1] = NULL;
+
 	gtk_set_locale ();
 	program = gnome_program_init (PACKAGE, VERSION,
-				      LIBGNOMEUI_MODULE, argc, argv,
+				      LIBGNOMEUI_MODULE, argc+1, new_argv,
 				      GNOME_PARAM_POPT_TABLE, popt_options,
 				      GNOME_PARAM_HUMAN_READABLE_NAME, _("Rhythmbox"),
 				      GNOME_PARAM_APP_DATADIR, DATADIR,
@@ -120,21 +135,20 @@ main (int argc, char **argv)
 #endif
 
 #ifdef HAVE_GSTREAMER	
-	/* rb currently does not work with esd + gnome event sounds enabled */
-	if (eel_gconf_get_boolean ("/desktop/gnome/sound/event_sounds")
-			&& eel_gconf_get_boolean ("/desktop/gnome/sound/enable_esd")
-	    && strstr (gst_gconf_get_string ("default/audiosink"), "esdsink"))
-		sound_events_borked = TRUE;
+	gst_init_with_popt_table (&argc, &argv, popt_options);
+	gst_control_init (&argc, &argv);
 #endif
+#ifdef WITH_MONKEYMEDIA
+	monkey_media_init (&argc, &argv);
+#endif
+
+	rb_debug_init (debug);
 	
 	gdk_threads_init ();
 
 	CORBA_exception_init (&ev);
 
-	/* Work around a GTK+ bug.  Inititalizing a dialog causes
-	 * a GTK+ warning, which then dies because it's a warning. */
-	if (!sound_events_borked)
-		rb_debug_init (debug);
+	rb_debug ("initializing Rhythmbox %s", VERSION);
 
 	rb_file_helpers_init ();
 	rb_string_helpers_init ();
@@ -154,21 +168,10 @@ main (int argc, char **argv)
 
 		rb_stock_icons_init ();
 
-		if (sound_events_borked) {
-			g_idle_add ((GSourceFunc) sound_error_dialog, NULL);
-		} else {
-			rb_shell = rb_shell_new ();
-			
-			g_object_set_data (G_OBJECT (rb_shell), "argv", argv);
-			g_object_set_data (G_OBJECT (rb_shell), "argc", GINT_TO_POINTER (argc));
-
-			g_object_set_data (G_OBJECT (rb_shell), "rb-shell-no-registration",
-					   GINT_TO_POINTER (no_registration));
-			g_object_set_data (G_OBJECT (rb_shell), "rb-shell-dry-run",
-					   GINT_TO_POINTER (dry_run));
-			
-			g_idle_add ((GSourceFunc) rb_init, rb_shell);
-		}
+		rb_shell = rb_shell_new (argc, argv, no_registration,
+					 no_update, dry_run, rhythmdb_file);
+		
+		g_idle_add ((GSourceFunc) rb_init, rb_shell);
 		
 		bonobo_main ();
 	} else {
@@ -180,33 +183,19 @@ main (int argc, char **argv)
 	/* cleanup */
 	CORBA_exception_free (&ev);
 
+#ifdef WITH_MONKEYMEDIA
+	monkey_media_shutdown ();
+#endif
 	rb_file_helpers_shutdown ();
 	rb_string_helpers_shutdown ();
+	if (object == NULL)
+		rb_stock_icons_shutdown ();
 
-	monkey_media_shutdown ();
+	g_strfreev (new_argv);
 
 	return 0;
 }
 
-
-static gboolean
-sound_error_dialog (gpointer unused)
-{
-	GtkWidget *dialog;
-
-	GDK_THREADS_ENTER ();
-
-	dialog = gtk_message_dialog_new (NULL, GTK_DIALOG_DESTROY_WITH_PARENT,
-					 GTK_MESSAGE_ERROR, GTK_BUTTONS_OK,
-					 _("Rhythmbox does not currently work if GNOME sound events are are enabled and esdsink is in use."));
-	gtk_dialog_run (GTK_DIALOG (dialog));
-
-	bonobo_main_quit ();
-
-	GDK_THREADS_LEAVE ();
-	return FALSE;
-}
-
 static gboolean
 rb_init (RBShell *shell)
 {
@@ -217,11 +206,9 @@ rb_init (RBShell *shell)
 	
 	rb_shell_construct (shell);
 
-	argv = (char **) g_object_get_data (G_OBJECT (shell), "argv");
-	argc = GPOINTER_TO_INT (g_object_get_data (G_OBJECT (shell), "argc"));
+	g_object_get (G_OBJECT (shell), "argc", &argc, "argv", &argv, NULL);
 
-	if (!g_object_get_data (G_OBJECT (shell), "rb-shell-no-registration")
-	    && (!g_object_get_data (G_OBJECT (shell), "rb-shell-dry-run")))
+	if (!no_registration && !dry_run)
 		rb_handle_cmdline (argv, argc, FALSE);
 	
 	GDK_THREADS_LEAVE ();
@@ -266,6 +253,32 @@ get_song_info (GNOME_Rhythmbox shell)
 	return song_info;
 }
 
+static void
+rb_toggle_shuffle (GNOME_Rhythmbox shell)
+{
+	Bonobo_PropertyBag pb;
+	gboolean shuffle;
+		
+	pb = GNOME_Rhythmbox_getPlayerProperties (shell, &ev);
+
+	g_return_if_fail (!BONOBO_EX (&ev));
+
+	shuffle = bonobo_pbclient_get_boolean (pb,
+					       "shuffle",
+					       &ev);
+	g_return_if_fail (!BONOBO_EX (&ev));
+
+		
+	bonobo_pbclient_set_boolean (pb,
+				     "shuffle",
+				     shuffle ? FALSE : TRUE,
+				     &ev);
+	g_return_if_fail (!BONOBO_EX (&ev));
+
+	bonobo_object_release_unref ((Bonobo_Unknown)pb, &ev);
+}
+
+
 static void
 rb_handle_cmdline (char **argv, int argc,
 		   gboolean already_running)
@@ -364,6 +377,11 @@ rb_handle_cmdline (char **argv, int argc,
 		grab_focus = FALSE;
 	}
 
+	if (shuffle)
+	{
+		rb_toggle_shuffle (shell);
+	}
+
 	for (i = 1; i < argc; i++)
 	{
 		char *tmp;
@@ -386,6 +404,10 @@ rb_handle_cmdline (char **argv, int argc,
 	if (focus)
 		grab_focus = TRUE;
 
+	if (already_running) {
+		gdk_notify_startup_complete ();
+	}
+
 	/* at the very least, we focus the window */
 	if (already_running == TRUE && grab_focus == TRUE)
 		GNOME_Rhythmbox_grabFocus (shell, &ev);
