@@ -55,6 +55,8 @@
 #include "rb-metadata.h"
 #include "rb-volume.h"
 #include "rb-remote.h"
+#include "rb-iradio-source.h"
+#include "rb-library-source.h"
 #include "eel-gconf-extensions.h"
 
 #include "rb-play-order.h"
@@ -70,6 +72,10 @@ typedef enum
 	PLAY_BUTTON_STOP
 } PlayButtonState;
 
+static const char* const state_to_play_order[2][2] =
+	{{"linear",	"linear-loop"},
+	 {"shuffle",	"random-by-age-and-rating"}};
+
 static void rb_shell_player_class_init (RBShellPlayerClass *klass);
 static void rb_shell_player_init (RBShellPlayer *shell_player);
 static GObject *rb_shell_player_constructor (GType type, guint n_construct_properties,
@@ -151,10 +157,16 @@ static void info_available_cb (RBPlayer *player,
 			       gpointer data);
 static void buffering_end_cb (RBPlayer *player, gpointer data);
 static void buffering_begin_cb (RBPlayer *player, gpointer data);
+static void buffering_progress_cb (RBPlayer *player, int progress, gpointer data);
+static void rb_shell_player_enable_buffering (RBShellPlayer *player);
 static void rb_shell_player_disable_buffering (RBShellPlayer *player);
 
-static void rb_shell_player_sync_play_order (RBShellPlayer *player);
+static void rb_shell_player_set_play_order (RBShellPlayer *player,
+					    const gchar *new_val);
 
+static void rb_shell_player_sync_play_order (RBShellPlayer *player);
+static void gconf_play_order_changed (GConfClient *client,guint cnxn_id,
+				      GConfEntry *entry, RBShellPlayer *player);
 
 #ifdef HAVE_MMKEYS
 static void grab_mmkey (int key_code, GdkWindow *root);
@@ -235,6 +247,9 @@ struct RBShellPlayerPrivate
 	GtkWidget *magic_button;
 
 	RBRemote *remote;
+
+	guint gconf_play_order_id;
+	guint gconf_state_id;
 };
 
 enum
@@ -243,7 +258,6 @@ enum
 	PROP_SOURCE,
 	PROP_COMPONENT,
 	PROP_TRAY_COMPONENT,
-	PROP_REPEAT,
 	PROP_PLAY_ORDER,
 	PROP_PLAYING,
 	PROP_BUFFERING,
@@ -279,19 +293,6 @@ static GObjectClass *parent_class = NULL;
 
 static guint rb_shell_player_signals[LAST_SIGNAL] = { 0 };
 
-static void
-gconf_key_changed (GConfClient *client,guint cnxn_id,
-		   GConfEntry *entry, RBShellPlayer *player)
-{
-	if (strcmp (CONF_STATE_REPEAT, entry->key) == 0) {
-		g_object_notify (G_OBJECT (player), "repeat");
-	} else if (strcmp (CONF_STATE_PLAY_ORDER, entry->key) == 0) {
-		rb_shell_player_sync_play_order (player);
-		g_object_notify (G_OBJECT (player), "play-order");
-	}
-}
-
-
 GType
 rb_shell_player_get_type (void)
 {
@@ -320,37 +321,6 @@ rb_shell_player_get_type (void)
 	return rb_shell_player_type;
 }
 
-void 
-rb_shell_player_set_repeat (RBShellPlayer *player, gboolean new_val)
-{
-	gboolean old_val;
-	/* Warning: there's probably a small race if the value stored in 
-	 * gconf is modified after the old_val != new_val test, but that
-	 * doesn't matter that much
-	 */
-	g_object_get (G_OBJECT (player), "repeat", &old_val, NULL);
-	if (old_val != new_val) {
-		/* The notify signal will be emitted by the gconf notifier */
-		eel_gconf_set_boolean (CONF_STATE_REPEAT, new_val);
-	}
-}
-
-
-void 
-rb_shell_player_set_play_order (RBShellPlayer *player, const gchar *new_val)
-{
-	const gchar *old_val;
-	/* Warning: there's probably a small race if the value stored in 
-	 * gconf is modified after the old_val != new_val test, but that
-	 * doesn't matter that much
-	 */
-	g_object_get (G_OBJECT (player), "play-order", &old_val, NULL);
-	if (strcmp (old_val, new_val) != 0) {
-		/* The notify signal will be emitted by the gconf notifier */
-		eel_gconf_set_string (CONF_STATE_PLAY_ORDER, new_val);
-	}
-}
-
 static void
 rb_shell_player_class_init (RBShellPlayerClass *klass)
 {
@@ -389,13 +359,6 @@ rb_shell_player_class_init (RBShellPlayerClass *klass)
 
 	/* If you change these, be sure to update the CORBA interface
 	 * in rb-shell.c! */
-	g_object_class_install_property (object_class,
-					 PROP_REPEAT,
-					 g_param_spec_boolean ("repeat", 
-							       "repeat", 
-							       "Whether repeat is enabled or not", 
-							       FALSE,
-							       G_PARAM_READABLE));
 	g_object_class_install_property (object_class,
 					 PROP_PLAY_ORDER,
 					 g_param_spec_string ("play-order", 
@@ -450,7 +413,7 @@ rb_shell_player_constructor (GType type, guint n_construct_properties,
 	RBShellPlayerClass *klass;
 	GObjectClass *parent_class;  
 
-	klass = RB_SHELL_PLAYER_CLASS (g_type_class_peek (type));
+	klass = RB_SHELL_PLAYER_CLASS (g_type_class_peek (RB_TYPE_SHELL_PLAYER));
 
 	parent_class = G_OBJECT_CLASS (g_type_class_peek_parent (klass));
 	player = RB_SHELL_PLAYER (parent_class->constructor (type, n_construct_properties,
@@ -472,8 +435,13 @@ rb_shell_player_init (RBShellPlayer *player)
 
 	player->priv->mmplayer = rb_player_new (&error);
 	if (error != NULL) {
-		rb_error_dialog (_("Failed to create the player: %s"), error->message);
-		g_error_free (error);
+		GtkWidget *dialog;
+		dialog = gtk_message_dialog_new (NULL, GTK_DIALOG_MODAL,
+						 GTK_MESSAGE_ERROR,
+						 GTK_BUTTONS_CLOSE,
+						 _("Failed to create the player: %s"),
+						 error->message);
+		gtk_dialog_run (GTK_DIALOG (dialog));
 		exit (1);
 	}
 
@@ -482,42 +450,45 @@ rb_shell_player_init (RBShellPlayer *player)
 
 	gtk_box_set_spacing (GTK_BOX (player), 12);
 
-	g_signal_connect (G_OBJECT (player->priv->mmplayer),
-			  "info",
-			  G_CALLBACK (info_available_cb),
-			  player);
+	g_signal_connect_object (G_OBJECT (player->priv->mmplayer),
+				 "info",
+				 G_CALLBACK (info_available_cb),
+				 player, 0);
 
-	g_signal_connect (G_OBJECT (player->priv->mmplayer),
-			  "eos",
-			  G_CALLBACK (eos_cb),
-			  player);
+	g_signal_connect_object (G_OBJECT (player->priv->mmplayer),
+				 "eos",
+				 G_CALLBACK (eos_cb),
+				 player, 0);
 
-	g_signal_connect (G_OBJECT (player->priv->mmplayer),
-			  "tick",
-			  G_CALLBACK (tick_cb),
-			  player);
+	g_signal_connect_object (G_OBJECT (player->priv->mmplayer),
+				 "tick",
+				 G_CALLBACK (tick_cb),
+				 player, 0);
 
-	g_signal_connect (G_OBJECT (player->priv->mmplayer),
-			  "error",
-			  G_CALLBACK (error_cb),
-			  player);
+	g_signal_connect_object (G_OBJECT (player->priv->mmplayer),
+				 "error",
+				 G_CALLBACK (error_cb),
+				 player, 0);
 
-	g_signal_connect (G_OBJECT (player->priv->mmplayer),
-			  "buffering_begin",
-			  G_CALLBACK (buffering_begin_cb),
-			  player);
+	g_signal_connect_object (G_OBJECT (player->priv->mmplayer),
+				 "buffering_begin",
+				 G_CALLBACK (buffering_begin_cb),
+				 player, 0);
 
-	g_signal_connect (G_OBJECT (player->priv->mmplayer),
-			  "buffering_end",
-			  G_CALLBACK (buffering_end_cb),
-			  player);
+	g_signal_connect_object (G_OBJECT (player->priv->mmplayer),
+				 "buffering_end",
+				 G_CALLBACK (buffering_end_cb),
+				 player, 0);
 
-	eel_gconf_notification_add (CONF_STATE_REPEAT,
-				    (GConfClientNotifyFunc)gconf_key_changed,
-				    player);
-	eel_gconf_notification_add (CONF_STATE_PLAY_ORDER,
-				    (GConfClientNotifyFunc)gconf_key_changed,
-				    player);
+	g_signal_connect_object (G_OBJECT (player->priv->mmplayer),
+				 "buffering_progress",
+				 G_CALLBACK (buffering_progress_cb),
+				 player, 0);
+
+	player->priv->gconf_play_order_id =
+		eel_gconf_notification_add (CONF_STATE_PLAY_ORDER,
+					    (GConfClientNotifyFunc)gconf_play_order_changed,
+					    player);
 
 	rb_shell_player_sync_volume (player);
 
@@ -590,9 +561,10 @@ rb_shell_player_init (RBShellPlayer *player)
 	gtk_container_add (GTK_CONTAINER (alignment), player->priv->volume_button);
 	gtk_box_pack_end (GTK_BOX (player), alignment, FALSE, TRUE, 0);
 
-	eel_gconf_notification_add (CONF_STATE,
-				    (GConfClientNotifyFunc) rb_shell_player_state_changed_cb,
-				    player);
+	player->priv->gconf_state_id = 
+		eel_gconf_notification_add (CONF_STATE,
+					    (GConfClientNotifyFunc) rb_shell_player_state_changed_cb,
+					    player);
 
 #ifdef HAVE_MMKEYS
 	/* Enable Multimedia Keys */
@@ -612,6 +584,9 @@ rb_shell_player_finalize (GObject *object)
 
 	g_return_if_fail (player->priv != NULL);
 
+	eel_gconf_notification_remove(player->priv->gconf_play_order_id);
+	eel_gconf_notification_remove(player->priv->gconf_state_id);
+
 	eel_gconf_set_float (CONF_STATE_VOLUME,
 			     rb_player_get_volume (player->priv->mmplayer));
 
@@ -619,6 +594,8 @@ rb_shell_player_finalize (GObject *object)
 
 	g_object_unref (G_OBJECT (player->priv->play_order));
 
+	gtk_object_destroy (GTK_OBJECT (player->priv->tooltips));
+
 	if (player->priv->remote != NULL)
 		g_object_unref (G_OBJECT (player->priv->remote));
 	
@@ -669,19 +646,19 @@ rb_shell_player_set_property (GObject *object,
 			RBEntryView *songs = rb_source_get_entry_view (player->priv->selected_source);
 			GList *extra_views = rb_source_get_extra_views (player->priv->selected_source);
 
-			g_signal_connect (G_OBJECT (songs),
-					  "changed",
-					  G_CALLBACK (rb_shell_player_entry_view_changed_cb),
-					  player);
-			g_signal_connect (G_OBJECT (songs),
-					  "entry-activated",
-					  G_CALLBACK (rb_shell_player_entry_activated_cb),
-					  player);
+			g_signal_connect_object (G_OBJECT (songs),
+						 "changed",
+						 G_CALLBACK (rb_shell_player_entry_view_changed_cb),
+						 player, 0);
+			g_signal_connect_object (G_OBJECT (songs),
+						 "entry-activated",
+						 G_CALLBACK (rb_shell_player_entry_activated_cb),
+						 player, 0);
 			for (; extra_views; extra_views = extra_views->next)
-				g_signal_connect (G_OBJECT (extra_views->data),
-						  "property-activated",
-						  G_CALLBACK (rb_shell_player_property_row_activated_cb),
-						  player);
+				g_signal_connect_object (G_OBJECT (extra_views->data),
+							 "property-activated",
+							 G_CALLBACK (rb_shell_player_property_row_activated_cb),
+							 player, 0);
 
 			g_list_free (extra_views);
 			
@@ -709,10 +686,6 @@ rb_shell_player_set_property (GObject *object,
 							     rb_shell_player_verbs,
 							     player);
 		break;
-	case PROP_REPEAT:
-		eel_gconf_set_boolean (CONF_STATE_REPEAT, 
-				       g_value_get_boolean (value));
-		break;
 
 	case PROP_PLAY_ORDER:
 		eel_gconf_set_string (CONF_STATE_PLAY_ORDER, 
@@ -744,9 +717,6 @@ rb_shell_player_get_property (GObject *object,
 	case PROP_TRAY_COMPONENT:
 		g_value_set_object (value, player->priv->tray_component);
 		break;
-	case PROP_REPEAT:
-		g_value_set_boolean (value, eel_gconf_get_boolean (CONF_STATE_REPEAT));
-		break;
 	case PROP_PLAY_ORDER:
 		g_value_set_string_take_ownership (value, eel_gconf_get_string (CONF_STATE_PLAY_ORDER));
 		break;
@@ -977,12 +947,68 @@ rb_shell_player_set_playing_entry (RBShellPlayer *player, RhythmDBEntry *entry)
 	rb_shell_player_sync_replaygain (player);
 }
 
+static void
+gconf_play_order_changed (GConfClient *client,guint cnxn_id,
+			  GConfEntry *entry, RBShellPlayer *player)
+{
+	rb_shell_player_sync_play_order (player);
+	g_object_notify (G_OBJECT (player), "play-order");
+}
+
+gboolean
+rb_shell_player_get_playback_state (RBShellPlayer *player,
+				    gboolean *shuffle,
+				    gboolean *repeat)
+{
+	int i, j;
+	char *play_order;
+
+	play_order = eel_gconf_get_string (CONF_STATE_PLAY_ORDER);
+	if (!play_order) {
+		g_critical (CONF_STATE_PLAY_ORDER " gconf key not found!");
+		return FALSE;
+	}
+
+	for (i = 0; i < G_N_ELEMENTS(state_to_play_order); i++)
+		for (j = 0; j < G_N_ELEMENTS(state_to_play_order[0]); j++)
+			if (!strcmp (play_order, state_to_play_order[i][j]))
+				goto found;
+
+	g_free (play_order);
+	return FALSE;
+
+found:
+	*shuffle = i > 0;
+	*repeat = j > 0;
+	g_free (play_order);
+	return TRUE;
+}
+
+static void 
+rb_shell_player_set_play_order (RBShellPlayer *player, const gchar *new_val)
+{
+	char *old_val;
+	g_object_get (G_OBJECT (player), "play-order", &old_val, NULL);
+	if (strcmp (old_val, new_val) != 0) {
+		/* The notify signal will be emitted by the gconf notifier */
+		eel_gconf_set_string (CONF_STATE_PLAY_ORDER, new_val);
+	}
+	g_free (old_val);
+}
+
+void
+rb_shell_player_set_playback_state (RBShellPlayer *player, gboolean shuffle, gboolean repeat)
+{
+	const char *neworder = state_to_play_order[shuffle ? 1 : 0][repeat ? 1 : 0];
+	rb_shell_player_set_play_order (player, neworder);
+}
+
 static void
 rb_shell_player_sync_play_order (RBShellPlayer *player)
 {
 	static char *current_play_order = NULL;
-
 	char *new_play_order = eel_gconf_get_string (CONF_STATE_PLAY_ORDER);
+
 	if (!new_play_order) {
 		g_critical (CONF_STATE_PLAY_ORDER " gconf key not found!");
 		new_play_order = g_strdup ("");
@@ -1028,25 +1054,43 @@ rb_shell_player_do_previous (RBShellPlayer *player)
 	RhythmDBEntry* entry;
 
 	if (player->priv->source != NULL) {
-		/* If we're in the first 2 seconds go to the previous song,
-		 * else restart the current one.
-		 */
-		if (rb_player_get_time (player->priv->mmplayer) < 3) {
-			rb_debug ("doing previous");
-			entry = rb_play_order_get_previous (player->priv->play_order);
-			if (entry) {
-				rb_play_order_go_previous (player->priv->play_order);
-				rb_shell_player_set_playing_entry (player, entry);
+
+		if (RB_IS_LIBRARY_SOURCE (player->priv->source)) {
+
+			rb_debug("previous algorithm for a library source");
+			
+			/* If we're in the first 2 seconds go to the previous song,
+			 * else restart the current one.
+			 */
+			if (rb_player_get_time (player->priv->mmplayer) < 3) {
+				rb_debug ("doing previous");
+				entry = rb_play_order_get_previous (player->priv->play_order);
+				if (entry) {
+					rb_play_order_go_previous (player->priv->play_order);
+					rb_shell_player_set_playing_entry (player, entry);
+				} else {
+					/* Is this the right thing to do when there's no previous song? */
+					rb_debug ("No previous entry, restarting song");
+					rb_player_set_time (player->priv->mmplayer, 0);
+					rb_header_sync_time (player->priv->header_widget);
+				}
 			} else {
-				/* Is this the right thing to do when there's no previous song? */
-				rb_debug ("No previous entry, restarting song");
+				rb_debug ("restarting song");
 				rb_player_set_time (player->priv->mmplayer, 0);
 				rb_header_sync_time (player->priv->header_widget);
 			}
-		} else {
-			rb_debug ("restarting song");
-			rb_player_set_time (player->priv->mmplayer, 0);
-			rb_header_sync_time (player->priv->header_widget);
+		}
+		else if (RB_IS_IRADIO_SOURCE (player->priv->source)) {
+		
+			rb_debug("previous algorithm for an iradio source");
+			
+			entry = rb_play_order_get_previous (player->priv->play_order);
+			
+			if (entry) {
+				rb_debug ("previous station found, doing previous");
+				rb_play_order_go_previous (player->priv->play_order);
+				rb_shell_player_set_playing_entry (player, entry);
+			}
 		}
 
 		rb_shell_player_jump_to_current (player);
@@ -1069,10 +1113,16 @@ rb_shell_player_auto_adjust_rating (RBShellPlayer *player, gboolean jumped)
 		return;
 	}
 	
+	/* only auto-rate songs because our algorithm only works for discrete tunes */
+	if (rhythmdb_entry_get_int (player->priv->db, current_entry, RHYTHMDB_PROP_TYPE) != RHYTHMDB_ENTRY_TYPE_SONG) {
+		return;
+	}
+	
 	entry_play_time = rb_shell_player_get_playing_time (player);
 	entry_duration = rb_shell_player_get_playing_song_duration (player);
 	entry_time_left = entry_duration - entry_play_time;
-	g_return_if_fail (entry_time_left >= 0);
+	if (entry_time_left <= 0)
+		return;
 	
 	/* sloppy end of song */
 	if (entry_time_left < 16)
@@ -1121,7 +1171,9 @@ rb_shell_player_auto_adjust_rating (RBShellPlayer *player, gboolean jumped)
 	}
 	
 	/* don't auto-rate songs 30 seconds or less */
-	if (eel_gconf_get_boolean (CONF_AUTO_RATE) && entry_duration > 30) {
+	if (eel_gconf_get_boolean (CONF_AUTO_RATE) && 
+	    rhythmdb_entry_get_boolean (player->priv->db, current_entry, RHYTHMDB_PROP_AUTO_RATE) &&
+	    entry_duration > 30) {
 		double old_rating, new_rating;
 		
 		/* get song's old rating */
@@ -1284,17 +1336,17 @@ rb_shell_player_cmd_stop (BonoboUIComponent *component,
 static void
 rb_shell_player_sync_control_state (RBShellPlayer *player)
 {
-	char *play_order;
-	
+	gboolean shuffle, repeat;
 	rb_debug ("syncing control state");
-	play_order = eel_gconf_get_string (CONF_STATE_PLAY_ORDER);
+
+	if (!rb_shell_player_get_playback_state (player, &shuffle,
+						 &repeat))
+		return;
+	
 	rb_bonobo_set_active (player->priv->component,
-			      CMD_PATH_SHUFFLE,
-			      strcmp (play_order, "linear") != 0);
-	g_free (play_order);
+			      CMD_PATH_SHUFFLE, shuffle);
 	rb_bonobo_set_active (player->priv->component,
-			      CMD_PATH_REPEAT,
-			      eel_gconf_get_boolean (CONF_STATE_REPEAT));
+			      CMD_PATH_REPEAT, repeat);
 }
 
 static void
@@ -1358,26 +1410,33 @@ rb_shell_player_shuffle_changed_cb (BonoboUIComponent *component,
 				    const char *state,
 				    RBShellPlayer *player)
 {
-	gboolean newval = rb_bonobo_get_active (component, CMD_PATH_SHUFFLE);
+	const char *neworder;
+	gboolean shuffle, repeat;
 	rb_debug ("shuffle changed");
 
-	if (newval) {
-		rb_shell_player_set_play_order (player, "shuffle");
-	} else {
-		rb_shell_player_set_play_order (player, "linear");
-	}
+	if (!rb_shell_player_get_playback_state (player, &shuffle, &repeat))
+		return;
+	shuffle = !shuffle;
+	neworder = state_to_play_order[shuffle ? 1 : 0][repeat ? 1 : 0];
+	rb_shell_player_set_play_order (player, neworder);
 }
 	
-static void rb_shell_player_repeat_changed_cb (BonoboUIComponent *component,
-					       const char *path,
-					       Bonobo_UIComponent_EventType type,
-					       const char *state,
-					       RBShellPlayer *player)
+static void
+rb_shell_player_repeat_changed_cb (BonoboUIComponent *component,
+				   const char *path,
+				   Bonobo_UIComponent_EventType type,
+				   const char *state,
+				   RBShellPlayer *player)
 {
-	gboolean newval = rb_bonobo_get_active (component, CMD_PATH_REPEAT);
+	const char *neworder;
+	gboolean shuffle, repeat;
 	rb_debug ("repeat changed");
 
-	rb_shell_player_set_repeat (player, newval);
+	if (!rb_shell_player_get_playback_state (player, &shuffle, &repeat))
+		return;
+	repeat = !repeat;
+	neworder = state_to_play_order[shuffle ? 1 : 0][repeat ? 1 : 0];
+	rb_shell_player_set_play_order (player, neworder);
 }
 
 static void
@@ -1569,6 +1628,7 @@ rb_shell_player_sync_with_source (RBShellPlayer *player)
 	else
 		rb_header_set_title (player->priv->header_widget, entry_title);
 	g_free (title);
+	
 	rb_header_set_playing_entry (player->priv->header_widget, entry);
 	rb_header_sync (player->priv->header_widget);
 }
@@ -1600,7 +1660,7 @@ rb_shell_player_sync_buttons (RBShellPlayer *player)
 		not_empty = TRUE;
 
 		/* Should these be up to the play order? */
-		have_previous = rb_player_get_uri (player->priv->mmplayer) != NULL;
+		have_previous = rb_player_opened (player->priv->mmplayer);
 		player->priv->have_previous_entry = (rb_entry_view_get_previous_entry (songs) != NULL);
 
 		have_next = rb_play_order_has_next (player->priv->play_order);
@@ -1633,14 +1693,12 @@ rb_shell_player_sync_buttons (RBShellPlayer *player)
 		rb_bonobo_set_sensitive (player->priv->component, CMD_PATH_PLAY, TRUE);
 
 	} else  {
-		if (rb_player_get_uri (player->priv->mmplayer) == NULL) {
+		if (rb_player_opened (player->priv->mmplayer)
+		    || player->priv->source == NULL
+		    || player->priv->source == player->priv->selected_source)
 			pstate = PLAY_BUTTON_PLAY;
-		} else {
-			if (player->priv->source == player->priv->selected_source)
-				pstate = PLAY_BUTTON_PLAY;
-			else
-				pstate = PLAY_BUTTON_STOP;
-		}
+		else
+			pstate = PLAY_BUTTON_STOP;
 
 		rb_bonobo_set_sensitive (player->priv->component, CMD_PATH_PLAY,
 					 rb_shell_player_have_first (player, source));
@@ -1687,10 +1745,10 @@ rb_shell_player_set_playing_source_internal (RBShellPlayer *player,
 
 	if (source != NULL) {
 		RBEntryView *songs = rb_source_get_entry_view (player->priv->source);
-		g_signal_connect (G_OBJECT (songs),
-				  "playing_entry_deleted",
-				  G_CALLBACK (rb_shell_player_playing_entry_deleted_cb),
-				  player);
+		g_signal_connect_object (G_OBJECT (songs),
+					 "playing_entry_deleted",
+					 G_CALLBACK (rb_shell_player_playing_entry_deleted_cb),
+					 player, 0);
 	}
 
 	if (player->priv->play_order)
@@ -1810,15 +1868,11 @@ eos_cb (RBPlayer *mmplayer, gpointer data)
 	GDK_THREADS_ENTER ();
 
 	if (player->priv->source != NULL) {
-		RBEntryView *songs = rb_source_get_entry_view (player->priv->source);
-
 		rb_debug ("updating play statistics");
 
-		rb_entry_view_freeze (songs);
 		rb_source_update_play_statistics (player->priv->source,
 						  player->priv->db,
 						  rb_shell_player_get_playing_entry (player));
-		rb_entry_view_thaw (songs);
 
 		switch (rb_source_handle_eos (player->priv->source))
 		{
@@ -1908,7 +1962,7 @@ info_available_cb (RBPlayer *mmplayer,
 	/* Sanity check, this signal may come in after we stopped the
 	 * player */
 	if (player->priv->source == NULL
-	    || !rb_player_get_uri (player->priv->mmplayer)) {
+	    || !rb_player_opened (player->priv->mmplayer)) {
 		rb_debug ("Got info_available but no playing source!");
 		return;
 	}
@@ -1991,23 +2045,34 @@ info_available_cb (RBPlayer *mmplayer,
 }
 
 static void
-buffering_begin_cb (RBPlayer *mmplayer,
-		    gpointer data)
+rb_shell_player_enable_buffering (RBShellPlayer *player)
 {
-	RBShellPlayer *player = RB_SHELL_PLAYER (data);
-	player->priv->buffering = TRUE;
-	g_object_notify (G_OBJECT (player), "buffering");
+	if (!player->priv->buffering) {
+		rb_debug("enabling buffering");
+		player->priv->buffering = TRUE;
+		g_object_notify (G_OBJECT (player), "buffering");
+	}
 }
 
 static void
 rb_shell_player_disable_buffering (RBShellPlayer *player)
 {
 	if (player->priv->buffering) {
+		rb_debug("disabling buffering");
 		player->priv->buffering = FALSE;
 		g_object_notify (G_OBJECT (player), "buffering");
 	}
 }
 
+static void
+buffering_begin_cb (RBPlayer *mmplayer,
+		    gpointer data)
+{
+	RBShellPlayer *player = RB_SHELL_PLAYER (data);
+	rb_debug ("got buffering_begin_cb");
+	rb_shell_player_enable_buffering (player);
+}
+
 static void
 buffering_end_cb (RBPlayer *mmplayer,
 		  gpointer data)
@@ -2019,6 +2084,30 @@ buffering_end_cb (RBPlayer *mmplayer,
  		rb_source_buffering_done (player->priv->source);
 }
 
+static void
+buffering_progress_cb (RBPlayer *mmplayer,
+		       int progress,
+		       gpointer data)
+{
+	RBShellPlayer *player = RB_SHELL_PLAYER (data);
+	rb_debug ("got buffering_progress_cb: %d", progress);
+
+	GDK_THREADS_ENTER();
+
+	if (progress == 100)
+	{
+		rb_shell_player_disable_buffering (player);
+		rb_shell_player_sync_with_source (player);
+		rb_shell_player_sync_buttons (player);
+	}	
+	else
+	{
+		rb_shell_player_enable_buffering (player);
+	}
+	
+	GDK_THREADS_LEAVE();
+}
+
 const char *
 rb_shell_player_get_playing_path (RBShellPlayer *shell_player)
 {
